@@ -23,6 +23,7 @@
 #' @param include_author If `TRUE`, results include document author(s)
 #' @param include_citations If `TRUE`, results include citations (CELEX-labelled)
 #' @param include_court_procedure If `TRUE`, results include type of court procedure and outcome
+#' @param include_ecli If `TRUE`, results include the ECLI identifier for court documents
 #' @param include_directory If `TRUE`, results include the Eur-Lex directory code
 #' @param include_sector If `TRUE`, results include the Eur-Lex sector code
 #' @param order Order results by ids
@@ -36,7 +37,7 @@
 #' elx_make_query(resource_type = "caselaw")
 #' elx_make_query(resource_type = "manual", manual_type = "SWD")
 
-elx_make_query <- function(resource_type = c("directive","regulation","decision","recommendation","intagr","caselaw","manual","proposal","national_impl","any"),
+elx_make_query <- function(resource_type = c("any","directive","regulation","decision","recommendation","intagr","caselaw","manual","proposal","national_impl"),
                            manual_type = "", directory = NULL, sector = NULL,
                            include_corrigenda = FALSE, include_celex = TRUE, include_lbs = FALSE,
                            include_date = FALSE, include_date_force = FALSE, include_date_endvalid = FALSE,
@@ -44,6 +45,7 @@ elx_make_query <- function(resource_type = c("directive","regulation","decision"
                            include_force = FALSE, include_eurovoc = FALSE, include_author = FALSE,
                            include_citations = FALSE, include_court_procedure = FALSE,
                            include_directory = FALSE, include_sector = FALSE,
+                           include_ecli = FALSE,
                            order = FALSE, limit = NULL){
 
   if (missing(resource_type)) stop("'resource_type' must be defined")
@@ -53,7 +55,7 @@ elx_make_query <- function(resource_type = c("directive","regulation","decision"
     stop("Please specify resource type manually (e.g. 'DIR', 'REG', 'JUDG').", call. = TRUE)
   }
 
-  if (!resource_type %in% c("caselaw","manual") & include_court_procedure == TRUE){
+  if (!resource_type %in% c("caselaw","manual","any") & include_court_procedure == TRUE){
     stop("Resource and variable requested are incompatible.", call. = TRUE)
   }
 
@@ -138,6 +140,12 @@ elx_make_query <- function(resource_type = c("directive","regulation","decision"
 
   }
 
+  if (include_ecli == TRUE){
+
+    query <- paste(query, "?ecli", sep = " ")
+
+  }
+
   if (include_author == TRUE){
 
     query <- paste(query, "?author", sep = " ")
@@ -404,6 +412,12 @@ elx_make_query <- function(resource_type = c("directive","regulation","decision"
 
   }
 
+  if (include_ecli == TRUE){
+
+    query <- paste(query, "OPTIONAL{?work cdm:case-law_ecli ?ecli.}")
+
+  }
+
   if (include_directory == TRUE){
 
     query <- paste(query, "OPTIONAL{?work cdm:resource_legal_is_about_concept_directory-code ?directory.}")
